// I2C_LCD.c
// ===============  Software I2C LCD  === Begin  ============================
// LCD declarations.
// Software I2C takes lots of time transmitting the signals bit by bit to LCD.
// SLC & SDA pins need to be defined. Software controlled


#define LCD_ADDR      (0x27 << 1)	  // =0x4E   // shifted I2C slave address 0x27 for LCD module. (0x27 << 1)
															// [slave address][R/W]=[0x27][0] address shift left 1 bit =0x4E


#define RS             0b00000001		//P0 - PCF8574T Pin connected to RS	0x01
#define RW            0b00000010		//P1 - PCF8574T Pin connected to RW	0x02
#define EN             0b00000100		//P2 - PCF8574T Pin connected to EN	0x04
#define LCD_LED	   0b00001000		//P3 - PCF8574T Pin connected to BACKLIGHT LED	0x08

/*
[RS][R/W][D7][D6][D5][D4][D3][D2][D1][D0]	  Set DDRAM adress, cursor position
  0	  0		 1	 AC6AC5 AC4AC3AC2AC1 AC0

0	0		1	0	0	0	0	0	0	0		0x80		= 0x00, column 1, line 1
0	0		1	1	0	0	0	0	0	0		0xC0		= 0x40, column 1, line 2
0	0		1	0	0	1	0	1	0	0		0x94		= 0x14, column 1, line 3
0	0		1	1	0	1	0	1	0	0		0xD4		= 0x54, column 1, line 4

*/

#define lcd_line1   0x80   // shifted LCD RAM address for line 1.
#define lcd_line2   0xC0   // shifted LCD RAM address for line 2
#define lcd_line3	0x94   // shifted LCD RAM address for line 3
#define lcd_line4   0xD4   // shifted LCD RAM address for line 4

//---------------------------------------------------------------------------------------------
void    delay_lcd_clk(unsigned int  delaynum)       //delay function for LCD clocking
{

	unsigned int	internalcounter;		// adjust counter for per delay clock

    while (delaynum > 0)        // delay per clk cycle.  about 20us +pulse on SCL pin, loop ~60us.
	 {
		 internalcounter=4;					// adjust for main delay per clock time
		 while (internalcounter > 0)       // delay. ~2.5us per loop at 16MHz clock Fosc
			internalcounter--;

		delaynum--;
	 }


}
//---------------------------------------------------------------------------------------------
void i2c_idle(void)		{	SDA = 1;	SCL = 1;	}

void i2c_start(void)
{
	SCL = 1;	delay_lcd_clk(1);
	SDA = 0;	delay_lcd_clk(1);
	SCL = 0;	delay_lcd_clk(1);
}

void i2c_restart(void)
{
	SDA = 1;	delay_lcd_clk(1);
	SCL = 1;	delay_lcd_clk(1);
	SDA = 0;	delay_lcd_clk(1);
}

void i2c_stop(void)
{
	SDA = 0;	delay_lcd_clk(1);
	SCL = 1;	delay_lcd_clk(1);
	SDA = 1;	delay_lcd_clk(1);
}

void i2c_ack(void)
{
	SDA = 0;	delay_lcd_clk(1);
	SCL = 1;	delay_lcd_clk(1);
	SCL = 0;	delay_lcd_clk(1);
}

void i2c_nak(void)
{
	SDA = 1;	delay_lcd_clk(1);
	SCL = 1;	delay_lcd_clk(1);
	SCL = 0;	delay_lcd_clk(1);
}

void i2c_send(unsigned char Data)
//unsigned char I2CSend(unsigned char Data)
{
//	unsigned char ack_bit;
	unsigned char bit1;

    for (bit1 = 0x80; bit1; bit1 >>= 1)	// shift until bit1 is 0. MSB first
	 {
		SCL = 0;			// start sending out bit
		delay_lcd_clk (1);
  		SDA = ((Data & bit1) ? 1 : 0);	// MSB shift out first. ? 1 or 0 result
		delay_lcd_clk(1);
		SCL = 1;			// set high to shift in bit to LCD
		delay_lcd_clk(1);
    }

	SCL = 0;
	delay_lcd_clk (1);
	//ack_bit = SDA;		// read ack.
	SCL = 1;
	delay_lcd_clk (1);
	SCL = 0;
//	return !ack_bit;
}
// ------------------------------------------------------------

//*************************************************************************************
/*
//  PCF8574T     LCD
//     P0        RS
//     P1        RW
//     P2        Enable
//     P3        LED Backlight (on/off)
//     P4        D4		// 4-bit mode
//     P5        D5
//     P6        D6
//     P7        D7


LCD pinout
 #		label		Description
 1		VSS		power ground.
 2		VDD		power +, +5V usually.
 3		V0			LCD contrast power.
 4		RS			Registerselectsignal.RS=0 Command,RS=1Data.
 5		R/W		Read/Writeselectsignal,R/W=1Read, R/W=0Write.
 6		E			Operationenablesignal, H->L triggers signal transfer.
 7		D0			Data bus bit 0.
 8		D1			Data bus bit 1.
 9		D2			Data bus bit 2.
 10	D3			Data bus bit 3.
 11	D4			Data bus bit 4.
 12	D5			Data bus bit 5.
 13	D6			Data bus bit 6.
 14	D7			Data bus bit 7.
 15	A			LED anode +, backlight
 16	K			LED Kathode -, backlight

[RS][RW][E] [D7][D6][D5][D4][D3][D2][D1][D0]		LCD data

[D7][D6][D5][D4] [BL][E][RW][RS]	I2C port

 */

void lcd_send_nibble(unsigned char data)	// send upper bits D7-D4 to LCD
{
		if (LCD_BL) data=data|EN|LCD_LED;	// set I2C pins [BL][E][RW][RS]
		else data=data|EN;										// EN set to high

		i2c_start();
		delay_lcd_clk(3);
		i2c_send(LCD_ADDR);		//the slave addresse
		delay_lcd_clk(3);
		i2c_send(data);			//setup the byte D7-D4
		delay_lcd_clk(3);
		data=data-4;       //toggle EN back to 0, clock in data to LCD. H->L of EN pin
		 i2c_send(data);		// change clock signal
		delay_lcd_clk(3);
		i2c_stop();
}

void lcd_clear(void)
{
    lcd_send_nibble(0x00);
    lcd_send_nibble(0x10);
    delay_lcd_clk(20);
}

void lcd_init(void)	// LCD init for 4 bit mode, 16x2 LCD.  also works for 20x4 LCD
{
    delay_lcd_clk(100);	// LCD power up delay_lcd_clk

	 // only D7-D4 used for data
		lcd_send_nibble(0x30);			// wake up chip #1 in 8 bit mode
		delay_lcd_clk(5);									// still works without this delay. added as extra delay for start
		lcd_send_nibble(0x30);		// wake up chip #2 in 8 bit mode
		lcd_send_nibble(0x30);		// wake up chip #3 in 8 bit mode

        lcd_send_nibble(0x20);			// start 4 bit mode

        lcd_send_nibble(0x20);			// set 4 bit mode
        lcd_send_nibble(0x80);		// 2 lines, 5x8 dots. 0x28

        lcd_send_nibble(0x00);			// Display On/Off, 0x0C
        lcd_send_nibble(0xC0);

        lcd_send_nibble(0x00);		// entry mode set, cursor shift right, no display shift, 0x06
        lcd_send_nibble(0x60);

        lcd_clear();		 //clears the display

}


void	lcd_linenum(unsigned char linenum)			//  start LCD cursor at line number
{
unsigned char data, address;

   switch(linenum)
   {
      case 1:  address= lcd_line1;     break;
      case 2:  address= lcd_line2;     break;
      case 3:  address= lcd_line3;   break;
      case 4:  address= lcd_line4;    break;
      default: address= lcd_line1;     break;
   }

   data=address & 0xF0;		// send high bits D7-D4
   lcd_send_nibble(data);
   data=address & 0x0F;		// send lower bits D3-D0
   data=data << 4;			// shift data to upper bits for sending nibble
   lcd_send_nibble(data);
}


void lcd_send_char(unsigned char in_data)	//Display a character on LCD screen.
{
 char data;

        data=in_data & 0xF0;		// send high bits D7-D4. also clears RS data pin, need to set 1=data
        data=data | RS;					//set RS pin to 1 = data
        lcd_send_nibble(data);
        data=in_data & 0x0F;		// send lower bits D3-D0
        data=data << 4;				// shift data to upper bits for sending nibble
        data=data | RS;				//set RS pin to 1 = data
        lcd_send_nibble(data);
}

// ---------------------------------------------------------------

//---------------------------------------------------------------------------------------------
void	WriteLCDLine(unsigned char line_num, const unsigned char *line_string)	// update LCD,  line # and string
{
// update LCD for line # and string

unsigned char	array_index=0;


switch (line_num)			// start cursor at line num position
{
	case 1: lcd_linenum(1);	break;		// start cursor at line 1.
	case 2: lcd_linenum(2);	break;		// start cursor at line 2.
	case 3: lcd_linenum(3);	break;		// start cursor at line 3.
	case 4: lcd_linenum(4);	break;		// start cursor at line 4.
	default: lcd_linenum(1);	break;		// default at line 1.
}

if ( (line_num > 0) && (line_num < 5) )				//  line number 1-4 only.
{
	while ( (line_string[array_index] != '\0') && (array_index < 20) )	// increment strlength until null char.
	{																							// limit character to 20 max for 20 char LCD.
		lcd_send_char(line_string[array_index]);
		array_index++;
	}
}

}
//---------------------------------------------------------------------------------------------


// ===============  Software I2C LCD  === End   ============================
//---------------------------------------------------------------------------------------------