/*=============================================
October 2, 2015
Dick Hsu
PIC12F1572-digitalPWM.c 	v1.0
PIC12F1572, set to 16MHz internal oscillator.
2kwords, 256 bytes RAM, 10bit ADC
ADC reference set to VDD.

 Digital PWM
 digital button settings.  up/down 5%, up/down 0.1%

 mode:
 0		off
 1		on

 
Update History:
Version	  Date:								Notes:
1.0			October 2, 2015			Initial program.  digital PWM


Pinout:
pin #1:	VDD		+5V power.
pin #2:	RA5		output,  I2C SDA
pin #3:	RA4		output, I2C SCL
pin #4:	RA3 	MCLR/VPP, programming pin. set to input. resistor pull high, button to ground
pin #5:	RA2		PWM3 output.  OUT3,
pin #6:	RA1		AN1, analog input, channel 2, VR2, (ICSPCLK), ADC button
pin #7:	RA0		AN0, analog input, channel 1, VR1, (ICSPDAT), ADC button
pin #8:	VSS     GND, negative power


 */

#include <xc.h>		//standard xc compiler header.
//#include <math.h>		// math functions
//#include <stdio.h>


// CONFIG1. setup device configurations.
#pragma config FOSC = INTOSC    //  internal osc.  I/O function on CLKIN pin.
#pragma config WDTE = OFF       // watchdog timer disabled.
#pragma config PWRTE = ON       // power up timer enabled, 65ms delay.
#pragma config MCLRE = OFF      // MCLR Pin Function Select, MCLR/VPP pin is digital input.
#pragma config CP = OFF         // Flash Program Memory Code Protection disabled
#pragma config BOREN = ON       // Brown-out Reset Enabled.
#pragma config CLKOUTEN = OFF   // CLKOUT function is disabled. I/O function on the CLKOUT pin)

// CONFIG2. setup device configurations.
#pragma config WRT = OFF        // Flash Memory Self-Write Protection disabled.
#pragma config PLLEN = OFF      // 4xPLL disabled. (for 32MHz FOSC)
#pragma config STVREN = OFF     // Stack Overflow or Underflow will not cause a Reset, disabled
#pragma config BORV = HI			// Brown-out Reset Voltage (Vbor), high trip point selected. 2.7V
#pragma config LPBOREN = OFF		// Low-Power BOR is disabled. (2.1V)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)


// pre-declarations, function prototypes.
//void	timestr(void);


//===========   Global Variables ==================


#define		HIGH		1					// define HIGH as 1.
#define  	LOW			0					// define Low as 0.
#define		SET			1					// set, set latch
#define		CLEAR		0				// clear, clear latch

// Port pin declarations
#define		BUTTON1_PORT        PORTAbits.RA3       // BUTTON1, resistor pull high, button to gnd.


#define     CH_BUTTON01				0		// AN0, 0.1%
#define     CH_BUTTON50							1		// AN1, 5%

/*
#define		DC0			0		// 0% duty cycle, off
#define		DC10		100		// 10% duty cycle
#define		DC20		200		// 10% duty cycle
#define		DC40		400		// 10% duty cycle
#define		DC60		600		// 10% duty cycle
#define		DC80		800		// 10% duty cycle
#define		DC100		1000		// 10% duty cycle
*/


// --------------------------------------- LCD I2C pin configuration-----------------------------------------------------
#define		SDA			PORTAbits.RA5		// I2C SDA data pin
#define		SCL			PORTAbits.RA4		// I2C SCK clock pin

bit		LCD_BL;			// LCD Backlight, single bit

// --------------------------------------- LCD -----------------------------------------------------

#define	LCDsize		16			// 16 characters


const unsigned char	initLCDline1[]="Digital PWM v1.0";
const unsigned char	initLCDline2[]="(C)2015  Onstate";


unsigned char	LCDline1[LCDsize+1];	//		// LCDsize+1, 1 for '\0' char
//unsigned char	LCDline2[LCDsize+1]=


// --------------------------------------- LCD -----------------------------------------------------


unsigned char	BUTTON1latch;
unsigned char	BUTTON1;		// button register

unsigned char	BUTTON2latch;
unsigned char	BUTTON2;		// up 5% button register, ADC 50

unsigned char	BUTTON3latch;
unsigned char	BUTTON3;		// down 5% button register


unsigned char	BUTTON4latch;
unsigned char	BUTTON4;		// up 0.1% button register, ADC 1

unsigned char	BUTTON5latch;
unsigned char	BUTTON5;		// down 0.1% button register



unsigned char digit[6];			// 	array for holding integer into separated digits. global variable
									//	int=[digit5][digit4][digit3][digit2][digit1][digit0],  digit0 is sign 0=+, 1=-.
/*
unsigned char, 1 byte 8bits. 0 to 255 max. positive number only
signed char,	1 byte 8 bits, -128 to 127
unsigned int,	2 byte 16 bits, 0 to 65535
signed int,		 2 byte 16 bits, -32768 to 32767
unsigned int	4 bytes 32 bits, 0 to 4,294,967,295
float				4 bytes, floating point, ~8 significant digits.

using unsigned variables (poistive number only) is twice faster than signed numbers
*/

unsigned char	hour=0, minute=0, second=0, decisecond=0;		// elapse time

unsigned char 	byteH, byteL;	//   high byte, low byte of an unsigned integer. for pwm variables, void itobyte(unsigned int intnum)


unsigned int 	adcdata[2]; // adc data registers.

unsigned int	brightness, ADCbutton50, ADCbutton01;

signed int		dutycycle;

unsigned char	mode;


unsigned int 	pwm3;	// pwm output numbers, using pwm3 only





// floating point, ~8 significant digits. 4 bytes size.
float floatnum1; 	// temp float number


// ===============================================================
void interrupt interruptfunction(void)
{
// --------------------------------------------------------------------
	if (TMR1IF)		// Timer1 interrupt flag set,
	{
		// ------------------------------------------------------
		TMR1IF=0;		// clear Timer1 interrupt flag so it can be set again. (must be cleared in software)

		TMR1H=0x3C;		// load values for 100ms timer.
		TMR1L=0xB2;

		// ------------------------------------------------------

		decisecond++;		// increment 100ms timer count
		if (decisecond >= 10)		// 100ms x 10 =1.00s
		{
			// ----------------------------------------
			decisecond=0;		// reset
			second++;		// increase 1 second
			// ----------------------------------------  // separates the seconds, minutes and hours --
			if (second >= 60)	{	minute++;	second=0; }		// increase minute  // reset second
			if (minute >= 60)	{	hour++;		minute=0; }		// increase hour // reset minute
			if (hour >= 24)	     {	 hour=0;	}						// reset hour
			// ----------------------------------------------------------------------------------------------------



		}

		// ------------------------------------------------------

	}	// if (TMR1IF)		// Timer1 interrupt flag set
// --------------------------------------------------------------------




}	// end void interrupt interruptfunction(void)
// ===============================================================



void itobyte(unsigned int intnum)			// integer-to-byte.
{ 	// splits an unsigned integer to a high byte and low byte. unsigned char byteH and byteL are global variables.
    byteL = (unsigned char) (intnum & 0x00FF); // select/mask lower 8 bits, cast unsigned char
    byteH = (unsigned char) (intnum >> 8); // converted high byte.  shift integer right 8 bits.
}

// ===============================================================

void	initPorts(void)		// initialize all PIC ports
{
    //----------------------- initialize internal oscillator  ---------------------------------------------------------------------
    //  OSCCON: OSCILLATOR CONTROL REGISTER
    //  [OSCCON]= [SPLLEN][IRCF<3:0>][-][SCS<1:0>]
    //                          0           1111        0   11
    // SPLLEN:Software PLL Enable bit, 0 = 4x PLL is disabled
    // IRCF<3:0>:Internal Oscillator Frequency Select bits, 1111= 16 MHz HF
    // SCS<1:0>:System Clock Select bits, 1x= Internal oscillator block

    SPLLEN = 0;
    IRCF3 = 1; // 1111= 16 MHz HF
    IRCF2 = 1;
    IRCF1 = 1;
    IRCF0 = 1;

    SCS1 = 1; // 1x= Internal oscillator block
    SCS0 = 1;


    //------------------------------- initialize analog channels -------------------------------------------------------------

    // initial default for PORTA is analog input.  Must set to digital I/O if required.
    // ANSELA: PORTA ANALOG SELECT REGISTER
    // [ANSELA]=[0][0][0][ANSA4][0][ANSA2][ANSA1][ANSA0]   // 1=analog input, 0=digital I/O

    ANSA0 = 1; //  analog input AN0
    ANSA1 = 1; //  analog input AN1
    ANSA2 = 0; //  digital I/O
    ANSA4 = 0; //  digital I/O

    //      TRISA: PORTA TRI-STATE REGISTER
    //  [TRISA]=[-][-][TRISA5][TRISA4][-(1)][TRISA2][TRISA1][TRISA0]

    TRISA0 = 1; // input, analog input AN0
    TRISA1 = 1; // input, analog input AN1
    TRISA2 = 0; // output, PWM3 output
    TRISA4 = 0; // output, output2.
    TRISA5 = 0; // output, output2.


}

// ===============================================================
void initPWM(void) 		// init PWM module setup.
{
	// each PWM module is individually controlled. 16 bit registers, 16 bit PWM
    //---------------------------------------------------------------------------------------------
    /*PWM1 register control.
     * PWMxCON:PWM CONTROL REGISTER
     * [PWMxCON]=[EN(0)][OE(0)][OUT(0)][POL(0)][MODE(00)][0][0]
     * PWM1CON  =  1           1       0           0           00          0  0
     * EN:PWM Module Enable bit,   1= Module is enabled, 0= Module is disabled
     * OE:PWM Output Enable bit,   1= PWM output pin enabled, 0= PWM output pin disabled
     * OUT:Output state of the PWM module
     * POL:PWM Output Polarity Control bit,     1= Output Polarity is inverted, 0= Output Polarity is normal
     * MODE<1:0>:PWM Mode Control bit,      00= Standard PWM mode
     */

 //   PWM1CON = 0b11000000; // set PWM1 register control.
//	 PWM2CON = 0b11000000; // set PWM2 register control.
    PWM3CON = 0b11000000; // set PWM3 register control.

    //---------------------------------------------------------------------------------------------
    /*PWMxCLKCON: PWM CLOCK CONTROL REGISTER
     *[PWMxCLKCON]=[0][PS<2:0>][0][0][CS<1:0>]
     * PWM1CLKCON=  0   011         0   0   01
     * PS<2:0>:Clock Source Prescaler Select bits,  011  = Divide clock source by 8, 16MHz/8=2MHz
     * CS<1:0>:Clock Source Select bits,    01= HFINTOSC
     */

 //   PWM1CLKCON = 0b00110001; // set  PWM1 clock to 2MHz
//    PWM2CLKCON = 0b00110001; // set  PWM2 clock to 2MHz
    PWM3CLKCON = 0b00110001; // set  PWM3 clock to 2MHz

    //---------------------------------------------------------------------------------------------
	// PWMxPHH: PWMx PHASE HIGH REGISTER
    // PWMxPHL: PWMx PHASE LOW REGISTER

 //   PWM1PHH = 0; // set phase to 0 to start at 0.
//    PWM1PHL = 0;

 //   PWM2PHH = 0; // set phase to 0 to start at 0.
 //   PWM2PHL = 0;

    PWM3PHH = 0; // set phase to 0 to start at 0.
    PWM3PHL = 0;
//---------------------------------------------------------------------------------------------
    // PWMxDCH: PWMx DUTY CYCLE HIGH REGISTER
    // PWMxDCL: PWMx DUTY CYCLE LOW REGISTER

//							(PWMxDC-PWMxPH)
// Duty Cycle = --------------------
//							PWMxPR

//    PWM1DCH = 0;	// PWM1 duty cycle, init to 0.
//    PWM1DCL = 0;	// PWM1 duty cycle, init to 0.

 //   PWM2DCH = 0;  // PWM2 duty cycle, init to 0.
 //   PWM2DCL = 0;  // PWM2 duty cycle, init to 0.

	PWM3DCH = 0;  // PWM3 duty cycle, init to 0.
    PWM3DCL = 0;  // PWM3 duty cycle, init to 0.

	//---------------------------------------------------------------------------------------------
    // PWMxPRH: PWMx PERIOD HIGH REGISTER
    // PWMxPRL: PWMx PERIOD LOW REGISTER
    // period set to 999 = 2KHz  999=0x03E7	// 999 is full on
//	              (PWMxPR+1)				999+1
// Period = --------------------------- = ----------=0.5ms=2KHz
//	               PWMxCLK						2MHz

//    PWM1PRH=0x03;
//    PWM1PRL=0xE7;

//	 PWM2PRH=0x03;
//    PWM2PRL=0xE7;

    PWM3PRH=0x03;
    PWM3PRL=0xE7;
	 //---------------------------------------------------------------------------------------------
    // PWMxTMRH: PWMx TIMER HIGH REGISTER
    // PWMxTMRL: PWMx TIMER LOW REGISTER
	// for center aligned mode.

//---------------------------------------------------------------------------------------------

    // PWMxOFH: PWMx OFFSET HIGH REGISTER
    // PWMxOFL: PWMx OFFSET LOW REGISTER

 //   PWM1OFH = 0; // set offset to 0
 //   PWM1OFL = 0;

 //   PWM2OFH = 0;
 //   PWM2OFL = 0;

    PWM3OFH = 0;
    PWM3OFL = 0;


//---------------------------------------------------------------------------------------------
    // PWMxLDCON: PWM RELOAD TRIGGER SOURCE SELECT REGISTER
    // [PWMxLDCON]=[LDA][LDT][0][0][0]0[0][LDS<1:0>]
    // LDA: Load Buffer Armed, 1 = Load OF, PH, DC and PR buffers at the end of the current period, reset after load.
    // LDT: Load Buffer on Trigger, 1 = Load buffers on trigger enabled, 0 = Load on trigger disabled

//   PWM1LD = 1; // set PWM1 registers to load new values, cleared by PIC when loaded
//  PWM2LD = 1; // set PWM2 registers to load new values, cleared by PIC when loaded
   PWM3LD = 1; // set PWM3 registers to load new values, cleared by PIC when loaded

}

// ===============================================================

void initADC(void)              // init ADC module
{
    /* set to internal voltage reference for ADC reference
    FVRCON: FIXED VOLTAGE REFERENCE CONTROL REGISTER
    FVRCON=[FVREN][FVRRDY][TSEN][TSRNG][CDAFVR<1:0>][ADFVR<1:0>]

    FVREN:Fixed Voltage Reference Enable bit
    1= Fixed Voltage Reference is enabled
    0= Fixed Voltage Reference is disabled

    ADFVR<1:0>:ADC Fixed Voltage Reference Selection bit
    11= ADC Fixed Voltage Reference Peripheral output is 4x (4.096V)
    10= ADC Fixed Voltage Reference Peripheral output is 2x (2.048V)
    01= ADC Fixed Voltage Reference Peripheral output is 1x (1.024V)
     */

    /*
    FVREN = 1; //	1= Fixed Voltage Reference is enabled for ADC
    ADFVR1 = 1; //	11= ADC Fixed Voltage Reference Peripheral output is 4x (4.096V)
    ADFVR0 = 1;
*/


    /*
    ADCON0: A/D CONTROL REGISTER 0
    ADCON0=[0][CHS<4:0>][GO/DONE][ADON]
    CHS<4:0>: Analog Channel Select bits
    00000 = AN0
    00001 = AN1
    00010 = AN2
    00011 = AN3

    GO/DONE:A/D Conversion Status bit
    1= Start A/D process, 0=A/D process finished.

    ADON:ADC Enable bit
    1= ADC is enabled,	0= ADC is disabled

     */
    CHS4 = 0;       // select 00000=AN0 as init
    CHS3 = 0;
    CHS2 = 0;
    CHS1 = 0;
    CHS0 = 0;

    ADON = 1; //	 enable the ADC converter.

    /*
    ADCON1: A/D CONTROL REGISTER 1
    ADCON1=[ADFM][ADCS<2:0>][0][0][ADPREF<1:0>]
    ADFM: A/D Result Format Select bit, 1= Right justified [000000ADRESH][ADRESL]
    ADCS<2:0>:A/D Conversion Clock Select bits, 	 Fosc/32 at 16MHz, 010=FOSC/32

     * ADPREF<1:0>: ADC Positive Voltage Reference Configuration bits
        00 = VRPOS is connected to VDD
        10 = VRPOS is connected to external VREF+ pin(1)
        11 = VRPOS is connected to internal Fixed Voltage Reference (FVR)
     *
     */

    ADFM = 1; // 1= Right justified [000000ADRESH][ADRESL]

    ADCS2 = 0; // Fosc/32 at 16MHz, 010=FOSC/32, convertion bit rate.
    ADCS1 = 1; // ~25uS to convert (12 Tad)
    ADCS0 = 0; // TACQ = 8.1uS to charger holding cap, 10k Ohm input impedance.

    ADPREF1 = 0;
    ADPREF0 = 0;


    ADIE = 0; // no ADC interrrupt,
    ADIF = 0; // clear the interrupt set flag.

}

// ===============================================================
void initTimer1(void)              // init Timer1 module
{

/*
 load value+2cycle delay+required count=65,536  TMR1IF overflow set
 4MHz/8prescale=0.5MHz, 0.5MHz*0.1second=50,000 count		100ms sampling rate

65536-50000 count=15536 counts required.

15536 + 2 count load register = 15538 This is the correct load value.
 It would be what the increment register count number would be after the load.
2 counts would have passed before starting count.
15538=0x3CB2

*/
	TMR1H=0x3C;		// load values for 100ms timer.
	TMR1L=0xB2;

	TMR1IF=0;		// clear timer1 interrupt flag.
/*
T1CON: TIMER1 CONTROL REGISTER
T1CON=[TMR1CS<1:0>][T1CKPS<1:0>][-][T1SYNC][-][TMR1ON]
TMR1CS<1:0>: Timer1 Clock Source Select bits, 00  = Timer1 clock source is instruction clock (FOSC/4)
 T1CKPS<1:0>: Timer1 Input Clock Prescale Select bits,  11  = 1:8 Prescale value
 !T1SYNC: Timer1 Synchronization Control bit,  1  = Do not synchronize asynchronous clock input
 TMR1ON: Timer1 On bit, 1  = Enables Timer1
*/

	TMR1CS1=0;		// 00  = Timer1 clock source is instruction clock (FOSC/4)= 16MHz/4=4MHz
	TMR1CS0=0;

	T1CKPS1=1;	// 11  = 1:8 Prescale value
	T1CKPS0=1;

//	T1SYNC=1;	// 1  = Do not synchronize asynchronous clock input

	TMR1ON=1;		// turn on timer1 module.


	TMR1GE=0;		// shut off gating control.


	GIE=1;		// enable global interrupts
	PEIE=1;		// enable peripheral interrupts
	TMR1IE=1;	// enable timer1 interrupt.


}


// ===============================================================

unsigned int getADC(unsigned char ANnum)        // get the ADC channel conversion
{
    /* do conversion on analog channels.  ANnum = analog channel number selected, returns the ADC converted number.
    ADC result registers: [000000ADRESH][ADRESL]
    CHS<4:0>: Analog Channel Select bits
    00000 = AN0
    00001 = AN1
    00010 = AN2
    00011 = AN3
     */

    unsigned int returnnum;


    CHS4 = 0;		// default at AN0;
    CHS3 = 0;
    CHS2 = 0;

switch (ANnum)
{
    case 0:		CHS1=0;	CHS0=0; 	break;	// AN0
    case 1:		CHS1=0;	CHS0=1; 	break;	// AN1
    case 2:		CHS1=1;	CHS0=0; 	break;	// AN2
    case 3:		CHS1=1;	CHS0=1; 	break;	// AN3
    default:    return 0; 	// default at 0, no reading, exit function immediately
}

    // setting up channel -> delay -> read ADC.
    for (returnnum = 0; returnnum < 10; returnnum++); // short delay to let channel selection settle before conversion.
    // counter=10 is 32uS delay at 16MHz

    GO_nDONE = 1;           // Start the ADC process
    while (GO_nDONE);       // Wait for conversion complete, 25us min. GO/DONE=0 when complete.
    returnnum = (ADRESH << 8) + ADRESL; // convert the 2 bytes to integer(10bits).  ADRESH shift left 8 bits=ADRESH*256
    return returnnum;           // return number.
}

// ===============================================================
void    PWM(unsigned int PWM3)        // sets the PWM3 module
{
      //-----------------------------------------------------------------------------------
/*
        itobyte(PWM1);      // PWM1, separate PWM into a high and low byte.
        PWM1DCH = byteH;
        PWM1DCL = byteL;
        PWM1LD = 1; // set PWM1 registers to load new values, cleared by PIC when loaded
        //-----------------------------------------------------------------------------------

        itobyte(PWM2);     // PWM2, separate PWM into a high and low byte.
        PWM2DCH = byteH;
        PWM2DCL = byteL;
        PWM2LD = 1; // set PWM2 registers to load new values, cleared by PIC when loaded
        //-----------------------------------------------------------------------------------
*/
         itobyte(PWM3); // PWM3, separate PWM into a high and low byte.
        PWM3DCH = byteH;
        PWM3DCL = byteL;
        PWM3LD = 1; // set PWM3 registers to load new values, cleared by PIC when loaded
        //-----------------------------------------------------------------------------------
}

// ===============================================================

void itochar(signed int intbreak)		// integer-to-character conversion,  signed int 16 bits.
{
// breaks down an integer number into individual digits. global array used as digit holders.
// unsigned char digit[6];  declared global variable.  unsigned int is 5 digits, 65535 max.

//extern unsigned char digit[6];		// set variable to external use. global declaration in main c.

unsigned char	digit_counter;

digit[0]='+';
digit[1]=digit[2]=digit[3]=digit[4]=digit[5]='0';			// clear all digits



	if (intbreak < 0)		// for signed negative numbers
	{
		digit[0]='-';		// negative number.
		intbreak= -(intbreak);	// make positive number
	}


	digit_counter=1;	// start at digit 1, digit[1].
	while (intbreak > 0)
	{
		digit[digit_counter] =(unsigned char) (intbreak % 10);	// modulo divide to get remainder
		digit[digit_counter] = digit[digit_counter] + '0';		// add ASCII '0' offset to number to convert to ASCII table.
		digit_counter++;												// increment to next digit
		intbreak = intbreak/10;						// shift to next digit
	}

}

// ===============================================================
//---------------------------------------------------------------------------------------------
#include "I2C_LCD.c"			// same as adding in the whole function here.
//---------------------------------------------------------------------------------------------
// ===============================================================
void    delay_ms(unsigned int  delaynum)       // program delay function 1ms
{
	unsigned int	internalcounter;

    while (delaynum > 0)        // delay per 1ms
	 {
		 internalcounter=397;					// counter=400 is 1ms delay at Fosc=16MHz, bit tested
														 // delay 1 ms. set internalcounter=397 taking into program delays
		 while (internalcounter > 0)       // delay. ~2.5us per loop at 16MHz clock Fosc
			internalcounter--;

		delaynum--;
	 }
}

// ===============================================================
void    get_ADCbutton01_input(void)    //  ADC channel for up/down button 0.1%
{

if (ADCbutton01 > 750) 	// up button,
	BUTTON4=SET;
else	BUTTON4=CLEAR;


if ( (ADCbutton01 < 500) && (ADCbutton01 > 100)	)	// down button
	BUTTON5=SET;
else	BUTTON5=CLEAR;


// ------------------------------ BUTTON 4----------------------------------------------------  up button

if ( (BUTTON4==SET) && (BUTTON4latch==CLEAR) )   // button pressed (low), not latched (read)
{
   // ----------- instructions --------------------------------------------------------
	dutycycle=dutycycle+1;		// increase 0.1%


   // ----------- instructions --------------------------------------------------------
   BUTTON4latch=SET;      // button read.  latch to indicate button read.
}	// if BUTTON4

// ---------------------------------------------------------
if ( (BUTTON4==CLEAR) && (BUTTON4latch==SET) )  // button released
   BUTTON4latch=CLEAR;      //  ok to take next button read.
// ------------------------------ BUTTON 4----------------------------------------------------


// ------------------------------ BUTTON 5 ----------------------------------------------------  down button

if ( (BUTTON5==SET) && (BUTTON5latch==CLEAR) )   // button pressed (low), not latched (read)
{
   // ----------- instructions --------------------------------------------------------
	dutycycle=dutycycle-1;		// decrease 0.1%


   // ----------- instructions --------------------------------------------------------
   BUTTON5latch=SET;      // button read.  latch to indicate button read.
}	// if BUTTON5

// ---------------------------------------------------------
if ( (BUTTON5==CLEAR) && (BUTTON5latch==SET) )  // button released
   BUTTON5latch=CLEAR;      //  ok to take next button read.
// ------------------------------ BUTTON 5 ----------------------------------------------------

// limit check

if (dutycycle >1000)		// keep all duty cycle 0-1000
	dutycycle=1000;

if (dutycycle < 0)
	dutycycle=0;


}

// ===============================================================
void    get_ADCbutton50_input(void)    //  ADC channel for up/down button 5%
{

if (ADCbutton50 > 750) 	// up button,
	BUTTON2=SET;
else	BUTTON2=CLEAR;


if ( (ADCbutton50 < 500) && (ADCbutton50 > 100)	)	// down button
	BUTTON3=SET;
else	BUTTON3=CLEAR;


// ------------------------------ BUTTON 2----------------------------------------------------  up button

if ( (BUTTON2==SET) && (BUTTON2latch==CLEAR) )   // button pressed (low), not latched (read)
{
   // ----------- instructions --------------------------------------------------------
	dutycycle=dutycycle+50;		// increase 5%



   // ----------- instructions --------------------------------------------------------
   BUTTON2latch=SET;      // button read.  latch to indicate button read.
}	// if BUTTON2

// ---------------------------------------------------------
if ( (BUTTON2==CLEAR) && (BUTTON2latch==SET) )  // button released
   BUTTON2latch=CLEAR;      //  ok to take next button read.
// ------------------------------ BUTTON 2----------------------------------------------------


// ------------------------------ BUTTON 3 ----------------------------------------------------  down button

if ( (BUTTON3==SET) && (BUTTON3latch==CLEAR) )   // button pressed (low), not latched (read)
{
   // ----------- instructions --------------------------------------------------------
	dutycycle=dutycycle-50;		// decrease 5%


   // ----------- instructions --------------------------------------------------------
   BUTTON3latch=SET;      // button read.  latch to indicate button read.
}	// if BUTTON3

// ---------------------------------------------------------
if ( (BUTTON3==CLEAR) && (BUTTON3latch==SET) )  // button released
   BUTTON3latch=CLEAR;      //  ok to take next button read.
// ------------------------------ BUTTON 3 ----------------------------------------------------

// limit check

if (dutycycle >1000)		// keep all duty cycle 0-1000
	dutycycle=1000;

if (dutycycle < 0)
	dutycycle=0;


}

// ===============================================================
void    get_button_input(void)    //  mode setting
{

	if (BUTTON1_PORT==LOW)		// low, buton pressed
		BUTTON1=SET;
	else	BUTTON1=CLEAR;

// --------------------------------------------------------------------------------------------------------
if ( (BUTTON1==SET) && (BUTTON1latch==CLEAR) )   // button pressed (low), not latched (read)
{
   // ----------- instructions --------------------------------------------------------
	if (mode == 1)		 // toggle modes
	{
		mode=0;
		LCD_BL=0;		// turn off LCD BL
	}
	else  
	{
		mode=1;			// on
		LCD_BL=1;		// turn on LCD BL
	}


   // ----------- instructions --------------------------------------------------------
   BUTTON1latch=SET;      // button read.  latch to indicate button read.
}	// if BUTTON1

// ---------------------------------------------------------
if ( (BUTTON1==CLEAR) && (BUTTON1latch==SET) )  // button released
   BUTTON1latch=CLEAR;      //  ok to take next button read.
// --------------------------------------------------------------------------------------------------------

}

// ===============================================================

// ====================	MAIN	==============================================

void main(void)
{

	initPorts();	// initialize all PIC ports
    initPWM();
    initADC();
	initTimer1();

    BUTTON1latch=0;

	pwm3=0;		// full on
	dutycycle=0;
	mode=1;		// default on

LCD_BL=1;
lcd_init();
lcd_clear();  //clear Display

WriteLCDLine(1, initLCDline1);
WriteLCDLine(2, initLCDline2);

 delay_ms(1500);		// delay

 lcd_clear();  //clear Display


//WriteLCDLine(1, LCDline1);
//WriteLCDLine(2, LCDline2);

    //	==================== Program Loop ===========================

    while (1) // loop program forever.
    {

		get_button_input();		// mode setting

//-----------------------------------------------------------------------------------
/*
        adcdata[CH_BUTTON01]= getADC(CH_BUTTON01);
		pwm3=adcdata[CH_BUTTON01];

		if (pwm3 > 1000)
			pwm3=1000;
*/
		adcdata[CH_BUTTON01]= getADC(CH_BUTTON01);
		ADCbutton01=adcdata[CH_BUTTON01];


		adcdata[CH_BUTTON50]= getADC(CH_BUTTON50);
		ADCbutton50=adcdata[CH_BUTTON50];

//-----------------------------------------------------------------------------------


// ---------------------------------------------------------------------------------------------

		if (mode==1)		// on
		{
			get_ADCbutton01_input();		// get ADC buttons. change only when in mode. 0.1%
			get_ADCbutton50_input();		// get ADC buttons. change only when in mode. 5.0%
			brightness=(unsigned int)(dutycycle);
		}
		else
			brightness=0;		// off


       PWM(brightness);       // sets the PWM output. PWM(pwm3)
		// Channel assignment. PWM3=OUT3

	   delay_ms(20);			// delay for whole program loop.  keeps program from running too fast.

// ---------------------------------------------------------------------------------------------

	itochar(brightness);
	LCDline1[0]=digit[4];
	LCDline1[1]=digit[3];
	LCDline1[2]=digit[2];
	LCDline1[3]='.';
	LCDline1[4]=digit[1];
	LCDline1[5]='%';
	LCDline1[6]='\0';


	if (mode ==0)		// off
	{
		LCDline1[0]='O';
		LCDline1[1]='F';
		LCDline1[2]='F';
		LCDline1[3]=' ';
		LCDline1[4]='0';
		LCDline1[5]='%';
		LCDline1[6]='\0';
	}


	   WriteLCDLine(1, LCDline1);

    } //while loop
    //	==================== Program Loop ===========================

} // main




